"""This module contains functions for extracting information from docker history."""

from .exceptions import ExpectingAShellCommand
from .nop import is_nop, strip_nop
from .shell import get_shell, is_shell, strip_shell


def parse_history(history: list) -> list:
    """Return reconstructed Dockerfile based on docker history."""
    return [parse_history_created_by(line["CreatedBy"]) for line in history]


def parse_history_created_by(created_by: str):
    """Parse 'CreatedBy' part of docker history and return comman that ran for that layer."""

    # current assumption is that all docker commands run shell, which can be NOP
    if not is_shell(created_by):
        raise ExpectingAShellCommand(
            f"Expecting '{created_by}' to contain '{get_shell}' but it does not."
        )

    if is_nop(created_by):
        created_by = strip_nop(created_by).strip()
    else:
        created_by = "RUN " + strip_shell(created_by).strip()

    return created_by



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dockerfile_recover', 'dockerfile_recover.parser']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0', 'docker>=5.0.0,<6.0.0', 'six>=1.16.0,<2.0.0']

entry_points = \
{'console_scripts': ['dockerfile-recover = dockerfile_recover.__main__:main']}

setup_kwargs = {
    'name': 'dockerfile-recover',
    'version': '0.0.1a1',
    'description': 'dockerfile-recover is a tool to reconstruct Dockerfile by reverse engineering a docker image',
    'long_description': '# dockerfile-recover\ndockerfile-recover is a tool to reconstruct Dockerfile by reverse engineering a docker image\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/purificant/dockerfile-recover',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""This module contains main codebase for dockerfile-recover tool."""

import click
import docker
from docker.errors import ImageNotFound
from docker.models.images import Image

from dockerfile_recover.parser import parse_history


@click.command(
    help="Recover Dockerfile by providing an image name similar to: docker pull <name>"
)
@click.argument("name")
def main(name):
    """Command line interface entrypoint."""

    client = docker.from_env()

    try:
        client.images.pull(name)
    except ImageNotFound:
        click.echo(f"{ImageNotFound.__name__}: {name}", err=True)
        return

    image: Image = client.images.get(name)

    history = image.history()
    history.reverse()

    lines = parse_history(history)
    click.echo("Recovered Dockerfile:")
    click.echo("")
    for line in lines:
        click.echo(line)

if __name__ == "__main__":
    main()



"""This module contains exceptions."""


class ParseException(Exception):
    """Base exception for all parsing errors."""


class ExpectingAShellCommand(ParseException):
    """
    Raised when docker histor command does not contain shell marker.
    The current assumption is that this never happens.
    """
